@@ -1,3 +1,31 @@
+/*
+ * Copyright (C) 1998 WIDE Project.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the project nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ */
 /*
  * Copyright (c) 1983, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
@@ -10,10 +38,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
  * 4. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
@@ -29,60 +53,58 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- */ 
+ */
 
 #if 0
 static char sccsid[] = "@(#)rcmd.c	8.3 (Berkeley) 3/26/94";
 #endif /* LIBC_SCCS and not lint */
 
-#define __FORCE_GLIBC
+#define __UCLIBC_HIDE_DEPRECATED__
 #include <features.h>
-
-#define __USE_GNU
-#include <ctype.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <errno.h>
-#include <alloca.h>
-#include <signal.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <pwd.h>
 #include <sys/param.h>
 #include <sys/poll.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
-#include <netdb.h>
+
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#ifdef __UCLIBC_HAS_THREADS__
-#undef __UCLIBC_HAS_THREADS__
-#warning FIXME I am not reentrant yet...
+#include <signal.h>
+#include <fcntl.h>
+#include <netdb.h>
+#include <unistd.h>
+#include <pwd.h>
+#include <errno.h>
+#include <stdio.h>
+#include <stdio_ext.h>
+#include <ctype.h>
+#include <string.h>
+#include <libintl.h>
+#include <stdlib.h>
+#ifdef __UCLIBC_HAS_WCHAR__
+#include <wchar.h>
 #endif
+#include <sys/uio.h>
+#include <bits/uClibc_alloc.h>
 
 
 /* some forward declarations */
 static int __ivaliduser2(FILE *hostf, u_int32_t raddr,
 			 const char *luser, const char *ruser, const char *rhost);
-static int iruserok2 (u_int32_t raddr, int superuser, const char *ruser, 
+static int iruserok2 (u_int32_t raddr, int superuser, const char *ruser,
 		      const char *luser, const char *rhost);
 
 
-int rcmd(ahost, rport, locuser, remuser, cmd, fd2p)
-     char **ahost;
-     u_short rport;
-     const char *locuser, *remuser, *cmd;
-     int *fd2p;
+int rcmd(char **ahost, u_short rport, const char *locuser, const char *remuser,
+		 const char *cmd, int *fd2p)
 {
-#ifdef __UCLIBC_HAS_THREADS__
+#ifdef __UCLIBC_HAS_REENTRANT_RPC__
 	int herr;
-        struct hostent hostbuf;
+	struct hostent hostbuf;
 	size_t hstbuflen;
 	char *tmphstbuf;
 #endif
-        struct hostent *hp;
+	struct hostent *hp;
 	struct sockaddr_in sin, from;
 	struct pollfd pfd[2];
 	int32_t oldmask;
@@ -92,23 +114,17 @@ int rcmd(ahost, rport, locuser, remuser, cmd, fd2p)
 
 	pid = getpid();
 
-#ifdef __UCLIBC_HAS_THREADS__
+#ifdef __UCLIBC_HAS_REENTRANT_RPC__
 	hstbuflen = 1024;
-#ifdef __UCLIBC_HAS_MMU__
-	tmphstbuf = alloca (hstbuflen);
-#else
-	tmphstbuf = malloc (hstbuflen);
-#endif
+	tmphstbuf = stack_heap_alloc(hstbuflen);
 
-	while (gethostbyname_r (*ahost, &hostbuf, tmphstbuf, 
+	while (gethostbyname_r (*ahost, &hostbuf, tmphstbuf,
 		    hstbuflen, &hp, &herr) != 0 || hp == NULL)
 	{
 	    if (herr != NETDB_INTERNAL || errno != ERANGE)
 	    {
 		__set_h_errno (herr);
-#ifndef __UCLIBC_HAS_MMU__
-		free(tmphstbuf);
-#endif
+		stack_heap_free(tmphstbuf);
 		herror(*ahost);
 		return -1;
 	    }
@@ -116,19 +132,11 @@ int rcmd(ahost, rport, locuser, remuser, cmd, fd2p)
 	    {
 		/* Enlarge the buffer.  */
 		hstbuflen *= 2;
-#ifdef __UCLIBC_HAS_MMU__
-		tmphstbuf = alloca (hstbuflen);
-#else
-		if (tmphstbuf) {
-		    free(tmphstbuf);
-		}
-		tmphstbuf = malloc (hstbuflen);
-#endif
+		stack_heap_free(tmphstbuf);
+		tmphstbuf = stack_heap_alloc(hstbuflen);
 	    }
 	}
-#ifndef __UCLIBC_HAS_MMU__
-	free(tmphstbuf);
-#endif
+	stack_heap_free(tmphstbuf);
 #else /* call the non-reentrant version */
 	if ((hp = gethostbyname(*ahost)) == NULL) {
 	    return -1;
@@ -136,7 +144,7 @@ int rcmd(ahost, rport, locuser, remuser, cmd, fd2p)
 #endif
 	pfd[0].events = POLLIN;
 	pfd[1].events = POLLIN;
-	
+
         *ahost = hp->h_name;
         oldmask = sigblock(sigmask(SIGURG)); /* __sigblock */
 	for (timo = 1, lport = IPPORT_RESERVED - 1;;) {
@@ -150,14 +158,14 @@ int rcmd(ahost, rport, locuser, remuser, cmd, fd2p)
 			sigsetmask(oldmask); /* sigsetmask */
 			return -1;
 		}
-		fcntl(s, F_SETOWN, pid); /* __fcntl */
+		fcntl(s, F_SETOWN, pid);
 		sin.sin_family = hp->h_addrtype;
-		bcopy(hp->h_addr_list[0], &sin.sin_addr,
+		memmove(&sin.sin_addr, hp->h_addr_list[0],
 		      MIN (sizeof (sin.sin_addr), hp->h_length));
 		sin.sin_port = rport;
 		if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) >= 0) /* __connect */
 			break;
-		(void)close(s); /* __close */
+		(void)close(s);
 		if (errno == EADDRINUSE) {
 			lport--;
 			continue;
@@ -175,7 +183,7 @@ int rcmd(ahost, rport, locuser, remuser, cmd, fd2p)
 			__set_errno (oerrno);
 			perror(0);
 			hp->h_addr_list++;
-			bcopy(hp->h_addr_list[0], &sin.sin_addr,
+			memmove(&sin.sin_addr, hp->h_addr_list[0],
 			      MIN (sizeof (sin.sin_addr), hp->h_length));
 			(void)fprintf(stderr, "Trying %s...\n",
 			    inet_ntoa(sin.sin_addr));
@@ -187,12 +195,12 @@ int rcmd(ahost, rport, locuser, remuser, cmd, fd2p)
 	}
 	lport--;
 	if (fd2p == 0) {
-		write(s, "", 1); /* __write */		    
+		write(s, "", 1);
 		lport = 0;
 	} else {
 		char num[8];
 		int s2 = rresvport(&lport), s3;
-		size_t len = sizeof(from);
+		socklen_t len = sizeof(from);
 
 		if (s2 < 0)
 			goto bad;
@@ -201,7 +209,7 @@ int rcmd(ahost, rport, locuser, remuser, cmd, fd2p)
 		if (write(s, num, strlen(num)+1) != strlen(num)+1) {
 			(void)fprintf(stderr,
 				      "rcmd: write (setting up stderr): %m\n");
-			(void)close(s2); /* __close */
+			(void)close(s2);
 			goto bad;
 		}
 		pfd[0].fd = s;
@@ -285,67 +293,56 @@ int rresvport(int *alport)
 	    return -1;
 	}
     }
-    
+
     return -1;
 }
+libc_hidden_def(rresvport)
 
-int	__check_rhosts_file = 1;
-char    *__rcmd_errstr;
+/* This needs to be exported ... while it is not a documented interface
+ * for rcp related apps, it's a required one that is used to control the
+ * rhost behavior.  Legacy sucks.
+ */
+int  __check_rhosts_file = 1;
 
-int ruserok(rhost, superuser, ruser, luser)
-	const char *rhost, *ruser, *luser;
-	int superuser;
+int ruserok(const char *rhost, int superuser, const char *ruser,
+			const char *luser)
 {
         struct hostent *hp;
 	u_int32_t addr;
 	char **ap;
-#ifdef __UCLIBC_HAS_THREADS__
+#ifdef __UCLIBC_HAS_REENTRANT_RPC__
 	size_t buflen;
 	char *buffer;
 	int herr;
 	struct hostent hostbuf;
 #endif
 
-#ifdef __UCLIBC_HAS_THREADS__
+#ifdef __UCLIBC_HAS_REENTRANT_RPC__
 	buflen = 1024;
-#ifdef __UCLIBC_HAS_MMU__
-	buffer = alloca (buflen);
-#else
-	buffer = malloc (buflen);
-#endif
+	buffer = stack_heap_alloc(buflen);
 
-	while (gethostbyname_r (rhost, &hostbuf, buffer, 
-		    buflen, &hp, &herr) != 0 || hp == NULL) 
+	while (gethostbyname_r (rhost, &hostbuf, buffer,
+		    buflen, &hp, &herr) != 0 || hp == NULL)
 	{
 	    if (herr != NETDB_INTERNAL || errno != ERANGE) {
-#ifndef __UCLIBC_HAS_MMU__
-		free(buffer);
-#endif
+		stack_heap_free(buffer);
 		return -1;
 	    } else
 	    {
 		/* Enlarge the buffer.  */
 		buflen *= 2;
-#ifdef __UCLIBC_HAS_MMU__
-		buffer = alloca (buflen);
-#else
-		if (buffer) {
-		    free(buffer);
-		}
-		buffer = malloc (buflen);
-#endif
+		stack_heap_free(buffer);
+		buffer = stack_heap_alloc(buflen);
 	    }
 	}
-#ifndef __UCLIBC_HAS_MMU__
-	free(buffer);
-#endif
+	stack_heap_free(buffer);
 #else
 	if ((hp = gethostbyname(rhost)) == NULL) {
 		return -1;
 	}
 #endif
 	for (ap = hp->h_addr_list; *ap; ++ap) {
-		bcopy(*ap, &addr, sizeof(addr));
+		memmove(&addr, *ap, sizeof(addr));
 		if (iruserok2(addr, superuser, ruser, luser, rhost) == 0)
 			return 0;
 	}
@@ -355,7 +352,7 @@ int ruserok(rhost, superuser, ruser, luser)
 
 /* Extremely paranoid file open function. */
 static FILE *
-iruserfopen (char *file, uid_t okuser)
+iruserfopen (const char *file, uid_t okuser)
 {
   struct stat st;
   char *cp = NULL;
@@ -364,7 +361,6 @@ iruserfopen (char *file, uid_t okuser)
   /* If not a regular file, if owned by someone other than user or
      root, if writeable by anyone but the owner, or if hardlinked
      anywhere, quit.  */
-  cp = NULL;
   if (lstat (file, &st))
     cp = "lstat failed";
   else if (!S_ISREG (st.st_mode))
@@ -387,7 +383,6 @@ iruserfopen (char *file, uid_t okuser)
   /* If there were any problems, quit.  */
   if (cp != NULL)
     {
-      __rcmd_errstr = cp;
       if (res)
 	fclose (res);
       return NULL;
@@ -407,17 +402,15 @@ iruserfopen (char *file, uid_t okuser)
  * Returns 0 if ok, -1 if not ok.
  */
 static int
-iruserok2 (raddr, superuser, ruser, luser, rhost)
-     u_int32_t raddr;
-     int superuser;
-     const char *ruser, *luser, *rhost;
+iruserok2 (u_int32_t raddr, int superuser, const char *ruser, const char *luser,
+		   const char *rhost)
 {
 	FILE *hostf = NULL;
 	int isbad = -1;
 
 	if (!superuser)
 		hostf = iruserfopen (_PATH_HEQUIV, 0);
-	
+
 	if (hostf) {
 		isbad = __ivaliduser2 (hostf, raddr, luser, ruser, rhost);
 		fclose (hostf);
@@ -432,26 +425,18 @@ iruserok2 (raddr, superuser, ruser, luser, rhost)
 		size_t dirlen;
 		uid_t uid;
 
-#ifdef __UCLIBC_HAS_THREADS__
+#ifdef __UCLIBC_HAS_REENTRANT_RPC__
 		size_t buflen = sysconf (_SC_GETPW_R_SIZE_MAX);
 		struct passwd pwdbuf;
-#ifdef __UCLIBC_HAS_MMU__
-		char *buffer = alloca (buflen);
-#else
-		char *buffer = malloc (buflen);
-#endif
+		char *buffer = stack_heap_alloc(buflen);
 
-		if (getpwnam_r (luser, &pwdbuf, buffer, 
+		if (getpwnam_r (luser, &pwdbuf, buffer,
 			    buflen, &pwd) != 0 || pwd == NULL)
 		{
-#ifndef __UCLIBC_HAS_MMU__
-			free(buffer);
-#endif
+			stack_heap_free(buffer);
 			return -1;
 		}
-#ifndef __UCLIBC_HAS_MMU__
-		free(buffer);
-#endif
+		stack_heap_free(buffer);
 #else
 		if ((pwd = getpwnam(luser)) == NULL)
 			return -1;
@@ -469,12 +454,12 @@ iruserok2 (raddr, superuser, ruser, luser, rhost)
 		seteuid (pwd->pw_uid);
 		hostf = iruserfopen (pbuf, pwd->pw_uid);
 		free(pbuf);
-		
+
 		if (hostf != NULL) {
 			isbad = __ivaliduser2 (hostf, raddr, luser, ruser, rhost);
 			fclose (hostf);
 		}
-		
+
 		seteuid (uid);
 		return isbad;
 	}
@@ -482,6 +467,7 @@ iruserok2 (raddr, superuser, ruser, luser, rhost)
 }
 
 /* This is the exported version.  */
+int iruserok (u_int32_t raddr, int superuser, const char * ruser, const char * luser);
 int iruserok (u_int32_t raddr, int superuser, const char * ruser, const char * luser)
 {
 	return iruserok2 (raddr, superuser, ruser, luser, "-");
@@ -500,6 +486,8 @@ int iruserok (u_int32_t raddr, int superuser, const char * ruser, const char * l
  * Returns 0 if ok, -1 if not ok.
  */
 int
+__ivaliduser(FILE *hostf, u_int32_t raddr, const char *luser, const char *ruser);
+int
 __ivaliduser(FILE *hostf, u_int32_t raddr, const char *luser, const char *ruser)
 {
 	return __ivaliduser2(hostf, raddr, luser, ruser, "-");
@@ -515,7 +503,7 @@ __icheckhost (u_int32_t raddr, char *lhost, const char *rhost)
 	int negate=1;    /* Multiply return with this to get -1 instead of 1 */
 	char **pp;
 
-#ifdef __UCLIBC_HAS_THREADS__
+#ifdef __UCLIBC_HAS_REENTRANT_RPC__
 	int save_errno;
 	size_t buflen;
 	char *buffer;
@@ -541,11 +529,11 @@ __icheckhost (u_int32_t raddr, char *lhost, const char *rhost)
 	}
 
 	/* Try for raw ip address first. */
-	if (isdigit (*lhost) && (long) (laddr = inet_addr (lhost)) != -1)
+	if (isdigit (*lhost) && (laddr = inet_addr (lhost)) != INADDR_NONE)
 		return negate * (! (raddr ^ laddr));
 
 	/* Better be a hostname. */
-#ifdef __UCLIBC_HAS_THREADS__
+#ifdef __UCLIBC_HAS_REENTRANT_RPC__
 	buflen = 1024;
 	buffer = malloc(buflen);
 	save_errno = errno;
@@ -559,7 +547,7 @@ __icheckhost (u_int32_t raddr, char *lhost, const char *rhost)
 	__set_errno (save_errno);
 #else
 	hp = gethostbyname(lhost);
-#endif /* __UCLIBC_HAS_THREADS__ */
+#endif /* __UCLIBC_HAS_REENTRANT_RPC__ */
 
 	if (hp == NULL)
 		return 0;
@@ -621,10 +609,8 @@ __isempty(char *p)
  * Returns 0 if positive match, -1 if _not_ ok.
  */
 static int
-__ivaliduser2(hostf, raddr, luser, ruser, rhost)
-	FILE *hostf;
-	u_int32_t raddr;
-	const char *luser, *ruser, *rhost;
+__ivaliduser2(FILE *hostf, u_int32_t raddr,	const char *luser,
+			  const char *ruser, const char *rhost)
 {
     register const char *user;
     register char *p;
@@ -697,8 +683,7 @@ __ivaliduser2(hostf, raddr, luser, ruser, rhost)
 	}
     }
 
-    if (buf != NULL)
-      free (buf);
+    free (buf);
 
     return retval;
 }
